#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>

#include "generalio.h"
#include "costfns.h"
#include "Globaloptions.h"

Globaloptions* Globaloptions::gopt = NULL;

void Globaloptions::parse_command_line(int argc,char** argv)
{
  if (argc < 3) {
    print_usage(argc,argv);
    exit(1);
  }

  int n=1;
  string arg;
  char first;
  
  while (n<argc) {
    arg=argv[n];
    if (arg.size()<1) { n++; continue; }
    first = arg[0];
    if (first!='-') {
      inputfname = arg;
      n++;
      cerr << "WARNING: change in option usage" << endl << endl;
      cerr << "To specify the input volume the option -in should be used" 
	   << endl << "Accepting the filename for now, but please update "
	   << "to new syntax in future." << endl << endl;
      continue;
    }
    // put options without arguments here
    if ( arg == "-help" ) {
      print_usage(argc,argv);
      exit(0);
    } else if ( arg == "-v" ) {
      verbose = 5;
      n++;
      continue;
    } else if ( arg == "-gdt" ) {
      gdtflag = 1;
      n++;
      continue;
    } else if ( arg == "-edge" ) {
      edgeflag = 1;
      n++;
      continue;
    } else if ( arg == "-hist" ) {
      histflag = 1;
      n++;
      continue;
    } else if ( arg == "-stats" ) {
      statflag = 1;
      n++;
      continue;
    } else if ( arg == "-mats" ) {
      tmpmatflag = 1;
      matflag = 1;
      n++;
      continue;
    } else if ( arg == "-plots" ) {
      tmpmatflag = 1;
      plotflag = 1;
      n++;
      continue;
    } else if ( arg == "-report" ) {
      no_reporting = 0;
      n++;
      continue;
    }
    
    if (n+1>=argc) 
      { 
	cerr << "Lacking argument to option " << arg << endl;
	break; 
      }

    // put options with 1 argument here
    if (arg == "-in") {
      inputfname = argv[n+1];
      n+=2;
      continue;
    } else if ( (arg == "-o") || (arg == "-out") ) {
      outputfname = argv[n+1];
      n+=2;
      continue;
    } else if ( arg == "-dof") {
      no_params = atoi(argv[n+1]);
      dof = atoi(argv[n+1]);
      n+=2;
      continue;
    } else if ( arg == "-verbose") {
      verbose = atoi(argv[n+1]);
      n+=2;
      continue;
    } else if ( arg == "-bins") {
      no_bins = atoi(argv[n+1]);
      n+=2;
      continue;
    } else if ( arg == "-refvol") {
      refnum = atoi(argv[n+1]);
      n+=2;
      continue;
    } else if ( arg == "-scaling") {
      scaling = atof(argv[n+1]);
      n+=2;
      continue;
    } else if ( arg == "-smooth") {
      smoothsize = atof(argv[n+1]);
      n+=2;
      continue;
    } else if ( arg == "-cost") {
      {
	string costarg = argv[n+1];
	if (costarg == "mutualinfo") {
	  maincostfn = MutualInfo;
	} else if (costarg == "corratio") {
	  maincostfn = CorrRatio;
	} else if (costarg == "woods") {
	  maincostfn = Woods;
	} else if (costarg == "normcorr") {
	  maincostfn = NormCorr;
	} else if (costarg == "normmi") {
	  maincostfn = NormMI;
	} else if (costarg == "leastsquares") {
	  maincostfn = LeastSq;
	} else {
	  if (!no_reporting) {
	    cerr << "Unrecognised cost function type: " << costarg << endl;
	    cerr << "Using the default (NormCorr)\n";
	  }
	}
      }
      n+=2;
      continue;
    }
  }

  if (inputfname.size()<1) {
    cerr << "Input filename not found\n\n";
    print_usage(argc,argv);
    exit(2);
  }
  
  if (outputfname.size()<1) {
    make_basename(inputfname);
    outputfname = inputfname + "_mcf";
    if (!no_reporting)
      cerr << "No output name specified. ";
  }
  
  if (!no_reporting)
    cerr << "Processed data will be saved as " << outputfname << ".(hdr/img)" << endl;
}

void Globaloptions::print_usage(int argc, char *argv[])
{
  cout << "Usage: mcflirt -in <infile> [options]\n\n"
       << "  Available options are:\n"
       << "        -out, -o <outputvol>               (default is mc_+original filename)\n"
       << "        -cost {mutualinfo,woods,corratio,normcorr,normmi,leastsquares}        (default is normcorr)\n"
       << "        -bins <number of histogram bins>   (default is "
       << no_bins << ")\n"
       << "        -dof  <number of transform dofs>   (default is "
       << dof << ")\n"
       << "        -refvol <number of reference volume> (default is no_vols/2)\n"
       << "        -scaling <num>                             (6.0 is default)\n"
       << "        -smooth <num>                      (1.0 is default - controls smoothing in cost function)\n"
       << "        -verbose <num>                     (0 is least and default)\n"
       << "        -gdt                               (run match on gradient images)\n"
       << "        -edge                              (run match on contour images)\n"
       << "        -stats                             produce variance and std. dev. images\n"
       << "        -mats                              save transformation matricies in subdirectory outfilename.mat\n"
       << "        -plots                             save transformation parameters in file outputfilename.par\n"
       << "        -report                            report progress to screen\n"
       << "        -help\n";
}







