#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>

#include "mjimage.h"
#include "generalio.h"
#include "interpolation.h"
#include "miscimfns.h"
#include "costfns.h"
#include "newmatap.h"
#include "newmatio.h"
#include "nrcode.h"

#include "Globaloptions.h"
#include "fudgefunctions.h"

using namespace MISCMATHS;
using namespace MISCIMFNS;
using namespace INTERPOLATION;
using namespace COSTFNS;
using namespace NEWMAT;
using namespace NUMREC;
using namespace MJIMAGE;
using namespace GENERALIO;

float costfn(const Matrix& uninitaffmat)
{
  Globaloptions& globalopts = Globaloptions::getInstance();

  Tracer tr("costfn");
  Matrix affmat = uninitaffmat * globalopts.initmat;  // apply initial matrix
  float retval = 0.0;
  switch (globalopts.maincostfn) 
    {
    case NormCorr:
      retval = 1.0 - fabs(normcorr(globalopts.impair,affmat));  // MAXimise corr
      break;
    case CorrRatio:
      retval = 1.0 - corr_ratio(globalopts.impair,affmat);  // MAXimise corr
      break;
    case Woods:
      retval = woods_fn(globalopts.impair,affmat);  // minimise variance/mean
      break;
    case MutualInfo:
      retval = -mutual_info(globalopts.impair,affmat);  // MAXimise info
      break;
    case NormMI:
      retval = -normalised_mutual_info(globalopts.impair,affmat);  // MAXimise
      break;
    default:
      cerr << "Invalid cost function type" << endl;
      break;
    }
  return retval;
}

void powell_opt(ColumnVector& params, int no_params, ColumnVector& param_tol, 
		int *no_its, float *fans, float (*costfunc)(float []), 
		int itmax)
{
  Globaloptions& globalopts = Globaloptions::getInstance();

  // sets up the initial parameters and calls the powell optimisation routine
  if (params.MaximumAbsoluteValue() < 0.001)  initialise_params(params);
  { 
    Matrix affmattst(4,4);
    vector2affine(params,no_params,affmattst);
    if (globalopts.verbose>=5) {
      cout << "Starting with : " << endl << affmattst;
    }
  }
  Matrix parambasis(no_params,no_params);
  set_param_basis(parambasis,no_params);
  float ptol[13];
  for (int i=1; i<=no_params; i++) { ptol[i] = param_tol(i); }
  
  // the optimisation call
  powell(params,parambasis,no_params,ptol,no_its, fans, costfunc, itmax);
}

void set_param_tols(ColumnVector &param_tol, int no_params)
{
      // Tolerances are: 0.57 degrees (0.005 radians), 0.2mm translation
      //    0.02 scale and 0.001 skew
  float diagonal[12]={0.005, 0.005, 0.005, 0.2, 0.2, 0.2, 0.002, 0.002, 0.002,
  		      0.001, 0.001, 0.001};
  if (param_tol.Nrows()<no_params) {
    param_tol.ReSize(no_params);
  }
  for (int i=1; i<=no_params; i++) {
    param_tol(i)=diagonal[i-1];
  }
}

int vector2affine(const ColumnVector& params, int n, const ColumnVector& centre,
		  Matrix& aff)
{
  Globaloptions& globalopts = Globaloptions::getInstance();

  if (n<=0) return 0;
  // order of parameters is 3 rotation + 3 translation + 3 scales + 3 skews
  // angles are in radians

  switch (globalopts.anglerep) 
    {
    case Euler:
      compose_aff(params,n,centre,aff,construct_rotmat_euler);
      break;
    case Quaternion:
      compose_aff(params,n,centre,aff,construct_rotmat_quat);
      break;
    default:
      cerr << "Invalid Rotation Representation" << endl;
      return -1;
    }
  return 0;
}  


int vector2affine(const ColumnVector& params, int n, Matrix& aff)
{
  Globaloptions& globalopts = Globaloptions::getInstance();
  return vector2affine(params,n,globalopts.impair->testvol.cog(),aff);
}


int vector2affine(const float params[], int n, Matrix& aff)
{
  ColumnVector vparams(12);
  for (int i=1; i<=n; i++) {
    vparams(i) = params[i];
  }
  return vector2affine(vparams,n,aff);
}

void set_param_basis(Matrix &parambasis, int no_params)
{
  parambasis = 0.0;
  for (int i=1; i<=no_params; i++) {
    parambasis(i,i)=1.0;
  }
}

void initialise_params(ColumnVector& params)
{
  Real paramsf[12] = {0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0};
  params << paramsf;
}


