#

#   iso - this changes the voxel size to make the image isotropic
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc makeiso { Folder Page zadd } {

    #{{{ find image properties

    set depth 1
    MxGetDimensions $Page width height depth

    puts "Dimensions=($width,$height,$depth)"

    set zscale 1
    MxGetScale $Page xscale yscale zscale

    puts "Voxel=($xscale,$yscale,$zscale)"

#}}}
    #{{{ setup scaling

    if { $xscale < $yscale } {
	set cubel $xscale
    } else {
	set cubel $yscale
    }	
    if { $zscale < $cubel } {
	set cubel $zscale
    }

    puts "Cubel side length=$cubel"

    set xscaling [ expr double($xscale) / $cubel ]
    set yscaling [ expr double($yscale) / $cubel ]
    set zscaling [ expr double($zscale) / $cubel ]

    puts "Scaling=($xscaling,$yscaling,$zscaling)"

#}}}
    #{{{ COMMENT setup new dimensions OLD

#    set newwidth  [ expr int( $width  * $xscaling ) ]
#    set newheight [ expr int( $height * $yscaling ) ]
#    set newdepth  [ expr int( $depth  * $zscaling ) ]
#
#    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
#    
#    if { $zadd > -0.001 } {
#	set zadd [ expr int( double($zadd) / $cubel ) ]
#	puts "Adding $zadd slices to top and bottom:"
#
#	set newdepth [ expr $newdepth + ( $zadd * 2 ) ]
#    } else {
#	set newdepth [ expr 0 - $zadd ]
#    }
#
#    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
#

#}}}
    #{{{ setup new dimensions

    set newwidth  [ expr int( $width  * $xscaling ) ]
    set newheight [ expr int( $height * $yscaling ) ]
    set newdepth  [ expr int( $depth  * $zscaling ) ]

    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
    
# this line took in zadd as mm rather than voxls - commented it out
#    set zadd [ expr int( double($zadd) / $cubel ) ]
    puts "Adding $zadd slices to top and bottom:"

    set newdepth [ expr $newdepth + ( $zadd * 2 ) ]

    puts "New Dimensions=($newwidth,$newheight,$newdepth)"

#}}}

    MxInterpolationMode BTL

    MxReslice $Page "{Cx [ expr ( $width - 1.0 ) / 2.0 ] } {Cy [ expr ( $height - 1.0 ) / 2.0 ] } \
	    {Cz [ expr ( $depth - 1.0 ) / 2.0 ] } {Rx 0} {Ry 0} {Rz 0} {Sx $xscaling} {Sy $yscaling} {Sz $zscaling} \
	    {Tx 0} {Ty 0} {Tz 0} {Dx $newwidth} {Dy $newheight} {Dz $newdepth } {RotMethod M}" IsoPage

    return $IsoPage
}
