#!/bin/sh

# This script runs the tcl script specified along with the
# default StartAPI.tcl file for the noninteractive case.
# For the interactive case a tixwish interpreter is started
# up (the user is expected to type source StartAPI.tcl and
# the user script). See History for more details.
# 
# Usage is 
#    runscript.sh 0 userscriptname 
#		where 0 specifies a noninterative mode
# or
#
#    runscript.sh 1 
#		where 1 specifies a interactive mode
#
# or
#    runscript.sh 2 userscriptname
#	        where 2 specified execute without shell
#
# Note that in the non interactive case if the script
# terminates due to error an error is not reported.
# Later on 0 and 1 case will be combined to one case

# History :
# found out that the ~/.tixwishrc file is not read in for tixwish
# in $PXHOME/bin/tixwish.
# the initial plan was to write out the script for both the
# interactive and noninteractive cases in to .tixwishrc
# so that the script could be run before making tixwish be in
# interactive mode. Another plan was to change tixwish binary
# to source a ~/.tixwishmedxrc file so that .tixwishrc was
# not disturbed (and an interactive shell could be obtained
# after running some initialization script). This required C changes
# But due to time shortage both these were abandoned and
# we accept a script only in the the non-interactive case right now
#

Usage() {
    echo "Usage: runscript.sh interactive [userscript]"
    exit 1
}

[ "$1" = "" ] && Usage

if [ "$1" = 0 ]
then
    [ "$2" = "" ] && Usage
fi

LocalCommand="set TheCommand dummycommand"
if [ "$#" != "3" ]
then
LocalCommand="set TheCommand \"$4\""
fi

LocalArgs="set TheArgs \" \""
if [ $# = 5 ]
then
LocalArgs="set TheArgs \"$5\""
fi

if [ _$PXHOME = _ ] 
then
    echo "PXHOME not set"
    exit 1
fi

if [ _$VEST_MESSAGE_PORT = _ ] 
then
    echo "VEST_MESSAGE_PORT not set"
    exit 1
fi

if [ _$DISPLAY = _ ] 
then
    echo "DISPLAY not set"
    exit 1
fi

PATH=${PATH}:/bin:$PXHOME/bin:/usr/bin/X11
export PATH

SCRIPTFILENAME="/tmp/userscript_$$"

INTERACTIVE=$1
APISCRIPT="$PXHOME/tcl/StartAPI.tcl" 

if [ $INTERACTIVE != 1 ]
then
    USERSCRIPT=$2
fi

trap 'rm -fr $SCRIPTFILENAME; exit' 1 2 15

if [ $INTERACTIVE = 1 ]
then 
    cd $PXHOME/tcl
    xterm -e "$PXHOME/bin/tixwish" 
    status=$?
else

cat - > $SCRIPTFILENAME << EOF
#!$PXHOME/bin/tixwish
wm withdraw .

set tracefilename ""
set tracechildset [ catch { set tracefilename \$env(TRACE_CHILDREN) } ]

if { \$tracechildset == 0 } {
    if { \$tracefilename != "" } {
	set id [ pid ]
	set f [ open \$tracefilename a+ ]
	puts \$f \$id 
	flush \$f
	close \$f
    }
} 

set TheFile $3
$LocalCommand
$LocalArgs

source $USERSCRIPT

exit
EOF

    if [ INTERACTIVE = 0 ]
    then
        xterm -e "$PXHOME/bin/tixwish" "$SCRIPTFILENAME" 
        status=$?
    else
	# INTERACTIVE is 2
	$PXHOME/bin/tixwish -f $SCRIPTFILENAME
	status=$?
    fi
    rm -fr $SCRIPTFILENAME
fi

exit $status

