#include <iostream>
#include "newimage.h"
#include "mjimage.h"

using namespace NEWIMAGE;

int main(int argc, char *argv[]) 
{

  // 1 - MJIMAGE (for loop)
  // 2 - Straight c++ pointers and new
  // 3 - Lazy sum
  // 4 - fast_const_iterator
  // 5 - const_iterator (position) (in for loop)
  // 6 - const_iterator (with reduced ROI)
  // 7 - manually inlined version of position iterator code
  // 8 - for loop
  // 9 - const_iterator (in while loop)
  // 10 - iterator (non-const)
  // 11 - fast_iterator (non-const)
  // 12 - for loop (const)

  float sum=0.0;
  int ver=0;

  if (argc<2) {
    ver = 1; 
  } else { 
    ver = atoi(argv[1]);
  }

  if (ver==1) {
    MJIMAGE::volume v4(200,250,200);
    v4 = 2;
    
    for (int n=0; n<20; n++) 
      for (int k=0; k<v4.zsize(); k++) {
	for (int j=0; j<v4.ysize(); j++) {
	  for (int i=0; i<v4.xsize(); i++) {
	    sum+= v4(i,j,k);
	  }
	}
      }
  }

  if (ver==2) {
    float *v5 = new float[200*250*200];
    float *pend = v5 + 200*250*200;
    for (float *it=v5; it!=pend; ++it) {  *it=2.0; } 
    for (int n=0; n<20; n++) {
      for (float *it=v5; it!=pend; ++it) {  sum+= *it; } 
    }
  }

  if (ver<=2) {
    cout << "Sum is : " << sum << endl;
    return 0;
  }
  
  volume<float> v3(200,250,200);
  const volume<float>& v3c(v3);
  v3 = 2;

  if (ver==3) {
    for (int n=0; n<20; n++) 
      sum += v3.sum();
  }

  if (ver==4) {
    for (int n=0; n<20; n++) 
      for (volume<float>::fast_const_iterator it=v3c.fbegin(), 
	     pend=v3c.fend();   it!=pend; ++it) 
	{  sum+= *it; }
  }


  if (ver==5) {
    for (int n=0; n<20; n++) 
      for (volume<float>::const_iterator it=v3c.begin(), 
	     pend=v3c.end();   it!=pend; ++it) 
	{  
	  sum+= *it; 
	}
  }

  
  if (ver==6) {
    int rx=0, ry=0, rz=0;
    v3.setROIlimits(1,1,1,100,100,100);
    v3.activateROI();
    for (int n=0; n<20; n++) 
      for (volume<float>::const_iterator it=v3c.begin(), 
	     pend=v3c.end();   it!=pend; ) 
	{  
	  sum+= *it; ++it;
	  it.getposition(rx,ry,rz);
	  //if (n==1) cout << "Position is ("<<rx<<","<<ry<<","<<rz<<")"<<endl;
	}
  }


  if (ver==7) {
    int lx0=0, lx1=199, ly0=0, ly1=249, lz0=0, lz1=199;
    int x=lx0,y=ly0,z=lz0;
    {
      for (int n=0; n<20; n++) {
	int RowAdjust=200+(lx0-lx1), 
	  SliceAdjust=200*250+200*(ly0-ly1)+(lx0-lx1);
	for (float *it = &(v3.value(lx0,ly0,lz0)), 
	       *pend =&(v3.value(lx1,ly1,lz1))+1; 
	     it!=pend; ) 
	  {  
	    sum+= *it; 
	    { x++; 
	    if (x>lx1) { x=lx0; y++; 
	    if (y>ly1) { y=ly0; z++; if (z>lz1) { ++it; } // end condition
	    else { it+=SliceAdjust; } } 
	    else { it+=RowAdjust; } } 
	    else { ++it;} }
	  }
      }
    }
  }


  if (ver==8) {
    for (int n=0; n<20; n++) 
      for (int k=v3.limits(2); k<=v3.limits(5); k++) {
	for (int j=v3.limits(1); j<=v3.limits(4); j++) {
	  for (int i=v3.limits(0); i<=v3.limits(3); i++) {
	    sum+= v3.value(i,j,k);
	  }
	}
      }
  }


  if (ver==9) {
    for (int n=0; n<20; n++) 
      {
	volume<float>::const_iterator it=v3c.begin(), pend=v3c.end();
	while (it!=pend) 
	  {  
	    sum+= *it;
	    ++it; 
	  }
      }
  }

  if (ver==10) {
    for (int n=0; n<20; n++) 
      for (volume<float>::iterator it=v3.begin(), 
	     pend=v3.end();   it!=pend; ++it) 
	{  
	  sum+= *it; 
	}
  }

  if (ver==11) {
    for (int n=0; n<20; n++) 
      for (volume<float>::fast_iterator it=v3.fbegin(), 
	     pend=v3.fend();   it!=pend; ++it) 
	{  
	  sum+= *it; 
	}
  }
    
  if (ver==12) {
    for (int n=0; n<20; n++) 
      for (int k=v3c.minz(); k<=v3c.maxz(); k++) {
	for (int j=v3c.miny(); j<=v3c.maxy(); j++) {
	  for (int i=v3c.minx(); i<=v3c.maxx(); i++) {
	    sum+= v3c.value(i,j,k);
	  }
	}
      }
  }


  if (ver==100) {
    try {
    // random testing code
    //v3.reinitialize(20,25,20);
    v3 = 2;
    v3.setROIlimits(0,0,0,19,24,19);
    v3.activateROI();
    cout << "Original sum is " << v3.sum() << endl;
    cout << " without modifying it is " << v3.sum() << endl;
    v3+=5;  // makes it 7
    cout << "Now it is " << v3.sum() << endl;
    cout << "Or " << v3.sum() << endl;
    v3-=8;  // makes it -1
    cout << "Now it is " << v3.sum() << endl;
    cout << " and v3(7,3,1) = " << v3(7,3,1) << endl;
    cout << "Or " << v3.sum() << endl;
    v3.deactivateROI();
    v3*=-1;  // makes it 1
    cout << "Now it is " << v3.sum() << endl;
    cout << " and v3(7,3,1) = " << v3(7,3,1) << endl;
    cout << "Or " << v3.sum() << endl;
    volume<float> v4 = v3;
    v4 = 2;
    cout << "v4 sum is " << v4.sum() << endl;
    v4.copyROIonly(v3);
    cout << "v4 sum is " << v4.sum() << endl;
    v4 *= 10; // v4 is 10
    cout << "v4 sum is " << v4.sum() << endl;
    cout << " and v4(7,3,1) = " << v4(7,3,1) << endl;
    volume<float> v5 = v4;
    v5 += v4; // v5 is 20
    cout << "v5 sum is " << v5.sum() << endl;
    cout << " and v5(7,3,1) = " << v5(7,3,1) << endl;
    v3 = v3 + v4;  // v3 is 10
    cout << "Now it is " << v3.sum() << endl;
    v3 += 1;  // v3 is 11
    cout << "Now it is " << v3.sum() << endl;
    v3 = v5;  // v3 is 20
    cout << "Now it is " << v3.sum() << endl;
    v3 = v5 + 5 + v4 ;  // v3 should become 35
    cout << "Now it is " << v3.sum() << endl;
    cout << " and v3(7,3,1) = " << v3(7,3,1) << endl;
    cout << "Or " << v3.sum() << endl;
    }
    catch (string msg) { cerr << msg << endl; throw; }
 } else {
    cout << "Sum is : " << sum << endl;
  }
    
}




