#include "options.h"

// $Id: parse.cc,v 1.1 2000/11/23 16:11:48 flitney Exp $

namespace Utilities {

  unsigned int OptionParser::parse_command_line(unsigned int argc, 
						char **argv) 
    throw(X_OptionError, X_UnknownOptions, X_MissingArguments)
  {
    unsigned int argcount = 1;
    bool unknown_opts = false;
    bool missing_args = false;

    while(argcount < argc) {
      unsigned int nmatches = 0;
      unsigned int increments = 1;
      
      string optstr(argv[argcount]), valstr;
      if(argcount + 1 < argc)
	valstr = string(argv[argcount+1]);
      else
	valstr = string("NO ARG");
      if(optstr[0] != '-')	// No more parsable options
	break;

      for(Options::iterator option = options_.begin();
	  option != options_.end();
	  option++) {
	if((*option)->matches(optstr))
	  {
	    nmatches++;
	    if((*option)->required()) {
	      if(valstr != "NO ARG") {
		(*option)->value(valstr);
		increments = 2;
	      } else {
		cerr << optstr << ": missing argument!" << endl;
		missing_args = true;
	      }
	    } else if((*option)->optional()) {
	      if(valstr[0] != '-') {
		(*option)->value(valstr);
		increments = 2;
	      } else {
		(*option)->value("NO ARG");
	      }
	    } else {
	      (*option)->value("NO ARG");
	    }
	  }
      }
      if(nmatches == 0) {
	cerr << optstr << ": unknown option!" << endl;
	unknown_opts = true;
      }
      argcount += increments;
    }
  
    if(unknown_opts && missing_args)
      throw X_OptionError();
    else if(unknown_opts)
      throw X_UnknownOptions();
    else if(missing_args)
      throw X_MissingArguments();

    return argcount;		// User should process any remaining args
  }

}
